;
; acat.asm
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;; acat [files]
;;
;;   Reads ascii/utf8 files and outputs them as petscii. Converts plain
;;   text, expands tabs, converts cr/lf and removes control characters.
;;
;    This file is part of the IDE64 shell utilities.
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

WILDEXP		= true

INPUT_LFN	= 1

last		= $2
ichars		= $fb
bufp		= $fd
buffsize	= $2000

banner		= "acat 0.3 by soci/singular"

		.include "lib.asm"

		.section code
start2		.proc
		ldx #<arglist
		ldy #>arglist
		jsr parseargs
		lda #<arglist
		sta p1
		lda #>arglist
		sta p1+1
		lda #<obuffer
		sta k+1
		lda #>obuffer
		sta k+2
		lda #0
		sta bufp

loop		ldy #0
		lda (p1),y
		beq _exit

		ldx p1
		ldy p1+1
		inx
		bne +
		iny
+		jsr kernal.setnam
		jsr addstring
		lda #INPUT_LFN
		ldx mydrive
		ldy #0
		sty last
		jsr kernal.setlfs
		jsr kernal.open
		bcs _nopen
		jsr onefile
		jsr kernal.clrchn
		lda #INPUT_LFN
		jsr kernal.close
_nopen		ldx kernal.stop
		cpx #$7f
		bne loop
_exit		jsr out.flush
		jsr kernal.clrchn
		lsr kernal.quote
		jmp exit

onefile		lda #<(ibuffer+buffsize-1)
		sta utf8.parse.get.pointer
		lda #>(ibuffer+buffsize-1)
		sta utf8.parse.get.pointer+1
		tsx
		stx utf8.parse.get.exit

		ldy #0
lp		ldx last
		cpx #"{tab}"
		bne +
		tya
		and #7
		cmp #7
		lda #32
		bcc k
		sta last
		gcs k

+		jsr utf8.topetscii
		sta last
		asl a
		cmp #32 << 1
		lda last
		bge nctrl

		cmp #"{tab}"
		beq lp
		cmp #"{cr}"
		beq k
		cmp #"{lf}"
		bne lp
		lda #"{cr}"
		cpx #"{cr}"
		beq lp
nctrl
k		sta obuffer
		inc k+1
		beq out
k2		iny
		eor #"{cr}"
		bne +
		tay
+		ldx kernal.stop
		cpx #$7f
		bne lp
		rts

out		.proc
		inc k+2
		ldx k+2
		cpx #>(obuffer+buffsize)
		bne k2
		jsr flush
		jmp k2

flush		pha
		tya
		pha

		ldx #STDOUT_LFN
		stx myout
		jsr kernal.chkout

		ldy #>obuffer
		sty bufp+1

		ldx k+1
		lda k+2
		sty k+2
		bcs _err
		sec
		sbc #>obuffer
		tay
		lda #bufp
		jsr write
_err
		pla
		tay
		pla
		rts
		.pend

utf8.parse.get	.proc
		inc pointer
		bne _next
		inc pointer+1
		lda pointer+1
		cmp #>(ibuffer+buffsize)
		bge _end

_next		inc ichars
		bne _nh
		inc ichars+1
		beq _exit

pointer		= *+1
_nh		lda ibuffer+buffsize-1
		rts

_end		txa
		pha
		tya
		pha
		lda #>ibuffer
		sta bufp+1
		sta pointer+1

		ldx #INPUT_LFN
		stx myin
		jsr kernal.chkin
		bcs _err

		ldx #<buffsize
		ldy #>buffsize
		lda #bufp
		jsr read
		txa
		eor #255
		sta ichars
		tya
		eor #255
		sta ichars+1
_err
		pla
		tay
		pla
		tax
		bcc _next
_exit
exit		= *+1
		ldx #0
		txs
		rts
		.pend

utf8		.binclude "lib/utf8.asm"

		.section bss
		.align 256
ibuffer		.fill buffsize
obuffer		.fill buffsize
		.send bss
		.pend
		.send code
