;
; cksum.asm
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;; cksum [files]
;;
;;   Calculates 32 bit CRC on files, and counts bytes. CRC algorithm
;;   is according to the POSIX standard. Can be used to check file
;;   integrity.
;;
;    This file is part of the IDE64 shell utilities.
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

WILDEXP		= true

INPUT_LFN	= 1

crctab0		= $c000
crctab1		= $c100
crctab2		= $c200
crctab3		= $c300

bufp            = $22
ichars		= $24
crc		= $fb
filelen		= $ac
ez		= $62
buffsize	= $4000

banner		= "cksum 0.2 by soci/singular"

		.include "lib.asm"

		.section code
start2		.proc
		ldx #<arglist
		ldy #>arglist
		jsr parseargs
		lda #<arglist
		sta p1
		lda #>arglist
		sta p1+1

		ldx #0
-		lda #0
		sta crc+2
		sta crc+1
		sta crc+0
		txa
		ldy #8
-		asl crc
		rol crc+1
		rol crc+2
		rol a
		bcc +
		eor #$04;$ed
		pha
		lda crc+2
		eor #$c1;$b8
		sta crc+2
		lda crc+1
		eor #$1d;$83
		sta crc+1
		lda crc+0
		eor #$b7;$20
		sta crc+0
		pla
+		dey
		bne -
		sta crctab3,x
		lda crc+2
		sta crctab2,x
		lda crc+1
		sta crctab1,x
		lda crc+0
		sta crctab0,x
		inx
		bne --

loop		ldy #0
		lda (p1),y
		beq _exit

		ldx p1
		ldy p1+1
		inx
		bne +
		iny
+		jsr kernal.setnam
		jsr addstring
		lda #INPUT_LFN
		ldx mydrive
		ldy #0
		jsr kernal.setlfs
		jsr kernal.open
		bcs _nopen
		ldx #INPUT_LFN
		stx myin
		jsr kernal.chkin
		bcs _ninput
		jsr onefile
		jsr kernal.clrchn
_ninput		lda #INPUT_LFN
		jsr kernal.close
_nopen		ldx kernal.stop
		cpx #$7f
		bne loop
_exit		geq exit

onefile		lda #0
		ldx #4
-		dex
		sta crc,x
		sta filelen,x
		bne -

		jsr doit
		bcc +
err             rts
+
		ldx #STDOUT_LFN
		jsr kernal.chkout
		bcs err

		ldx #3
-		lda filelen,x
		sta $26,x
		dex
		bpl -

		lda filelen+3
-		ora filelen+2
		ora filelen+1
		ora filelen
		beq ki
		lda filelen
                jsr crcgen2
		lda filelen+1
		sta filelen
		lda filelen+2
		sta filelen+1
		lda filelen+3
		sta filelen+2
		lda #0
		sta filelen+3
		geq -
ki
		ldx #3
-		lda crc,x
		eor #255
		sta crc,x
		dex
		bpl -

		ldx #crc
		jsr outnum
		ldx #$26
		jsr outnum
		ldy #0
-		cpy kernal.filename_len
		beq +
		lda (kernal.filename),y
		jsr kernal.chrout
		iny
		bne -
+		lda #"{cr}"
		jmp kernal.chrout

crcgen          .macro
		eor crc+3
		tax
		lda crctab3,x
		eor crc+2
		sta crc+3
		lda crctab2,x
		eor crc+1
		sta crc+2
		lda crctab1,x
		eor crc
		sta crc+1
		lda crctab0,x
		sta crc
                .endm

crcgen2		.proc
		#crcgen
                rts
		.pend

outnum		.proc
		php
		sei
		sed
		lda #0
		ldy #4
-		sta ez,y
		dey
		bpl -

		jsr do
		jsr do
		jsr do
		jsr do

		plp

		ldx #128
		ldy #251
-		lda (ez-251) & $ffff,y
		jsr outhex
		iny
		bne -
		txa
		bpl +
		lda #$30
		jsr kernal.chrout
+		lda #32
		jmp kernal.chrout

do		lda $03,x
		sec
		rol a
lp		pha
		ldy #4
-		lda ez,y
		adc ez,y
		sta ez,y
		dey
		bpl -
		pla
		asl a
		bne lp
		dex
		rts
		.pend

outhex		.proc
		pha
		lsr a
		lsr a
		lsr a
		lsr a
		jsr o
		pla
o		and #$0f
		bne +
		inx
		bpl +
		rts
+		ora #$30
		tax
		jmp kernal.chrout
		.pend

doit		.proc
		lda kernal.status
		beq +
		clc
stopped		rts

+		lda #<buffer
		sta bufp
		lda #>buffer
		sta bufp+1
		sta pointer+2

		ldx #<buffsize
		ldy #>buffsize
		lda #bufp
		jsr read
		bcs stopped
		txa
		adc filelen
		sta filelen
                tya
		adc filelen+1
		sta filelen+1
                bcc +
		inc filelen+2
                bne +
		inc filelen+3
+
                inx
		stx ichars
		sty ichars+1

                ldy #0
-               dec ichars+1
                bmi last
pointer		lda buffer,y
                #crcgen
                iny
		bne pointer
		ldx kernal.stop
		cpx #$7f
		beq stopped
                inc bufp+1
		inc pointer+2
                gne -

last            dec ichars
		beq doit
		lda (bufp),y
		jsr crcgen2
                iny
                jmp last
		.pend

		.section bss
		.align 256
buffer		.fill buffsize
		.send bss
		.pend
		.send code

