;
; lib.asm
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
; Library routines for shell utilities
;
;    This file is part of the IDE64 shell utilities.
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

kernal		.binclude "kernal.asm"

p1		= $14

STDOUT_LFN	= 3

		*=$0801

		.dsection code
		.section code
		.word (+), 2020
		.null $9e, format("%d", start)
+		.word 0
		.send code

		.dsection data

		.union
		.struct
		jmp start2
		.text "ide64shell"
		.ends
		.struct
		.dsection bss
		.ends
		.endu
arglist
;---------------------------------
		.section code
start		.proc
		lda #<argtxt
		ldy #>argtxt
		jsr $ab1e
		ldy #0
		sty $7a
		lda #>$200
		sta $7b
		dey
-		iny
		jsr kernal.chrin
		sta ($7a),y
		eor #"{cr}"
		bne -
		sta ($7a),y
		iny
		iny
		sta ($7a),y
		lda #"{cr}"
		jsr kernal.chrout
		jmp start2

		.section data
argtxt		.text "{cr}", banner, "{cr}"
		.null "{cr}arguments:"
		.send data
		.pend
;
; Parse arguments
;
; "quoted text"
; >tofile
; >>appendfile
;
parseargs	.proc
		stx p1
		sty p1+1
		lda kernal.device
		sta mydrive
		jsr kernal.clall
		lda #STDOUT_LFN
		tax
		ldy #1
		jsr kernal.setlfs
		jsr kernal.open
next		ldy #0
		sty redir+1
		sty append+1
		.if WILDEXP
		sty wild+1
		.fi
		ldx #0
		lda ($7a,x)
		bne +
end		txa
		sta (p1,x)
		rts

-		jsr get
+		beq end
		cmp #" "
		beq -
		cmp #">"
		bne +
		ror redir+1
		jsr get
		cmp #">"
		bne +
		ror append+1
		jsr get
+
-		cmp #'"'
		beq q
		iny
		sta (p1),y
		.if WILDEXP
		cmp #"/"
		bne +
		lsr wild+1
+		cmp #"?"
		beq vc
		cmp #"*"
		bne qb
vc		ror wild+1
		.fi
qb		jsr get
		beq kesz
		cmp #" "
		bne -
		geq kesz

-		iny
		sta (p1),y
q		jsr get
		beq kesz
		cmp #'"'
		bne -
		geq qb

kesz            jsr redir
                jmp next

redir		ldx #0
		.if WILDEXP
                bpl wild
		.else
		bpl termstring
                .fi
		lsr redir+1
append		lda #0
		php
		bpl napp
		lda #","
		iny
		sta (p1),y
		lda #"a"
		iny
		sta (p1),y
napp		lsr append+1
		tya
		ldx p1
		ldy p1+1
		inx
		bne +
		iny
+		jsr kernal.setnam
		lda #STDOUT_LFN
		jsr kernal.close
		ldx #1
		plp
		bpl +
		inx
+		stx kernal.secondary
		lda mydrive
		sta kernal.device
		jmp kernal.open

		.if WILDEXP
wild		lda #0
		bpl termstring
		lda #"$"
		sta (p1,x)
		iny
		tya
		ldx p1
		ldy p1+1
		jmp dirlist
                .fi

get		inc $7a
		bne +
		inc $7b
+		lda ($7a,x)
		rts
		.pend

termstring      .proc
		tya
                ldx #0
		sta (p1,x)
		geq addstring
		.pend

addstring       .proc
		sec
		adc p1
		sta p1
		bcc +
		inc p1+1
+		rts
                .pend

		.section data
mydrive		.proc
		.byte 0
		.pend
myin		.proc
		.byte 0
		.pend
myout		.proc
		.byte STDOUT_LFN
		.pend
		.send data
;
; List directory content
;
dirlist		.proc
DIRLIST_LFN	= 16

		jsr kernal.setnam
		ldy #0
		sty pathend
-		lda (kernal.filename),y
		cmp #","
		bne +
		lda #"="
		sta (kernal.filename),y
+		cmp #"/"
		bne +
		sty pathend
+		iny
		cpy kernal.filename_len
		blt -
		lda #DIRLIST_LFN
		ldx mydrive
		tay
		jsr kernal.setlfs
		jsr kernal.open
		bcs err2
		ldx #DIRLIST_LFN
		jsr kernal.chkin
		bcs err

		ldy #6
		jsr skip
readline
-		lda kernal.status
		bne end
		jsr kernal.chrin
		bne -
		ldy #4
		jsr skip
-		lda kernal.status
		bne end
		jsr kernal.chrin
		beq end
		cmp #'"'
		bne -
pathend		= *+1
-		cpy #0
		bge +
		iny
		lda (kernal.filename),y
		sta (p1),y
		glt -
+		ldx #0
-		jsr kernal.chrin
		beq end
		iny
		inx
		sta (p1),y
		cmp #'"'
		bne -
		lda #","
		sta (p1),y
-		jsr kernal.chrin
		inx
		cpx #19
		blt -
		iny
		sta (p1),y
		cpx #21
		blt -
                jsr termstring
		jmp readline

end		jsr kernal.clrchn
err		lda #DIRLIST_LFN
		jsr kernal.close
err2		ldy #0
		tya
		sta (p1),y
		rts

skip		jsr kernal.chrin
		dey
		bne skip
		rts
		.pend
;
; Execute command on command channel, and print result
;
readerr		.proc
COMMAND_LFN	= 15

		jsr kernal.setnam
		lda #COMMAND_LFN
		ldx mydrive
		tay
		jsr kernal.setlfs
		jsr kernal.open
		bcc +
		rts

+		ldx #COMMAND_LFN
		jsr kernal.chkin
		bcs err
		jsr kernal.chrin
		pha
		lsr a
		cmp #"0" >> 1
		bne +
		pla
-		jsr kernal.chrin
		lda kernal.status
		beq -
		gne en
+
		ldx #STDOUT_LFN
		jsr kernal.chkout
		pla
		bcs err
		gcc +

-		jsr kernal.chrin
+		jsr kernal.chrout
		lda kernal.status
		beq -
en		jsr kernal.clrchn
err		lda #COMMAND_LFN
		jmp kernal.close
		.pend
;
; Blockread with fallback
;
read		.proc
		stx _readlp+1
		sty _rh+1
		tax
		lda $00,x
		sta _pointer+1
		lda $01,x
		sta _pointer+2
		jsr is_ide64
unsupported	= *+1
		ora #0
		bne _noblk
		txa
		ldx _readlp+1
		jsr $def4
		bcc _exit
		inc unsupported

		ldx myin
		jsr kernal.chkin
		bcs _exit

_noblk		ldx #0
		ldy #0
		sty kernal.status

_readlp		lda #0
		bne +
_rh		lda #0
		beq _exit
		dec _rh+1
+		dec _readlp+1
		jsr kernal.chrin
_pointer	sta $ffff,x
		lda kernal.status
		bne _exit2
		inx
		bne _readlp
		iny
		inc _pointer+2
		bne _readlp
_exit		rts

_exit2		eor #$40
		bne +
		inx
		bne +
		iny
+		rts
		.pend
;
; Blockwrite with fallback
;
write		.proc
		stx _readlp+1
		sty _rh+1
		tax
		lda $00,x
		sta _pointer+1
		lda $01,x
		sta _pointer+2
		jsr is_ide64
unsupported	= *+1
		ora #0
		bne _noblk
		txa
		ldx _readlp+1
		jsr $def1
		bcc _exit
		inc unsupported

		ldx myout
		jsr kernal.chkout
		bcs _exit

_noblk		ldx #0
		ldy #0
		sty kernal.status

_readlp		lda #0
		bne +
_rh		lda #0
		beq _exit
		dec _rh+1
+		dec _readlp+1
_pointer	lda $ffff,x
		jsr kernal.chrout
		lda kernal.status
		bne _exit
		inx
		bne _readlp
		iny
		inc _pointer+2
		bne _readlp
_exit		rts
		.pend
;
; IDE64 check
;
is_ide64	.proc
		lda $de60
		eor #"i"
		bne +
		lda $de61
		eor #"d"
		bne +
		lda $de62
		eor #"e"
+		rts
		.pend
;
; Exit
;
exit		.proc
		lda #STDOUT_LFN
		jsr kernal.close
		lda mydrive
		sta kernal.device
		rts
		.pend
;
; Hex printer
;
printhex	.proc
		pha
		lsr a
		lsr a
		lsr a
		lsr a
		jsr +
		pla
		and #$0f
+		cmp #10
		blt +
		adc #6
+		adc #$30
		jmp kernal.chrout
		.pend
		.send code
