;
; unlzo.asm
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;; unlzo [files]
;;
;;   Extracts multiple lzo files.
;;
;    This file is part of the IDE64 shell utilities.
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

		.virtual $1000
buffer		.fill $c000
		.endv

WILDEXP		= true

INPUT_LFN	= 1
OUTPUT_LFN	= 5

oldt		= $02
t2		= $ae
p2		= $22
t		= $24
p3		= $fd

banner		= "unlzo 0.1 by soci/singular"

		.include "lib.asm"

		.section code
copymem		.proc
		php
		sei
		lda $01
		pha
		lda #$34
		sta $01
		stx _lp+2
		sty _lp+5
		ldy #$30
		ldx #0
_lp		lda buffer,x
		sta $d000,x
		inx
		bne _lp
		inc _lp+2
		inc _lp+5
		dey
		bne _lp
		pla
		sta $01
		plp
		rts
		.pend

lzo		.binclude "lib/lzo.asm"

dolzo		.proc
		ldx #>buffer
		ldy #>$d000
		jsr copymem
		inc restore+1
		jsr lzo
restore		lda #0
		beq +
		ldx #>$d000
		ldy #>buffer
		jsr copymem
		lsr restore+1
+		rts
		.pend

start2		.proc
		jsr dolzo.restore

		.cerror * > buffer

		ldx #<arglist
		ldy #>arglist
		jsr parseargs
		lda #<arglist
		sta p1
		lda #>arglist
		sta p1+1

		dec $01
loop		ldy #0
		lda (p1),y
		beq _exit

		ldx p1
		ldy p1+1
		inx
		bne +
		iny
+		jsr kernal.setnam
		jsr addstring
		lda #INPUT_LFN
		ldx mydrive
		ldy #0
		sty read.unsupported
		sty write.unsupported
		jsr kernal.setlfs
		jsr kernal.open
		bcs _nopen
		jsr onefile
		lda #INPUT_LFN
		jsr kernal.close
_nopen		ldx kernal.stop
		cpx #$7f
		bne loop
_exit		inc $01
		jmp exit

onefile
;----------------------------------------------
		ldx #INPUT_LFN
		stx myin
		jsr kernal.chkin

		ldx #-size(magic)
-		jsr kernal.chrin
		cmp magic-(<-size(magic)),x
		bne unk2
		inx
		bne -

		.section data
magic		.byte $89, $4c, $5a, $4f, $00, $0d, $0a, $1a, $0a
		.send data

		jsr kernal.chrin	; version
		tax
		jsr kernal.chrin
		tay
		cpx #>$900
		blt unk2		; too old

		jsr kernal.chrin	; libversion
		jsr kernal.chrin

		cpy #<$940
		txa
		sbc #>$940
		php
		blt +

		jsr kernal.chrin	; version needed to extract
		jsr kernal.chrin
+
		jsr kernal.chrin	; method
		sta header.method

		plp
		php
		blt +
		jsr kernal.chrin	; level
+
		ldx #4
-		jsr kernal.chrin	; flags
		dex
		sta header.flags,x
		sta lzo.flags,x
		bne -

		lda header.flags+1
		and #$08		; filter?
		beq +
		ldx #4
-		jsr kernal.chrin	; filter
		dex
		bne -
+
		plp
		ldx #8
		blt +
		ldx #12
+
-		jsr kernal.chrin	; mode, mtime
		dex
		bne -

		jsr kernal.chrin	; filename length
		tax
		beq +
		ldy #0
-		jsr kernal.chrin
		sta header.name,y
		iny
		dex
		bne -
+		lda #0
		sta header.name,y

		ldx #4
-		jsr kernal.chrin	; checksum
		dex
		bne -

		bit header.flags	; extra field
		bvs unk2

		lda header.method
		cmp #1
		blt unk2
		cmp #3+1
		blt supported

unk2		ldx #STDOUT_LFN
		jsr kernal.chkout

		lda #<unknowntxt
		ldy #>unknowntxt
		inc $1
		jsr $ab1e
		dec $1
		jmp kernal.clrchn

unknowntxt	.section data
		.null "{cr}unknown format.{cr}"
		.send data

supported
		jsr nametopetscii

		ldx #STDOUT_LFN
		jsr kernal.chkout
		bcs errstou

		ldy #0
-		lda #1
		sta kernal.quote
		lda header.name,y
		beq +
		jsr kernal.chrout
		iny
		bne -
+		lda #"{cr}"
		jsr kernal.chrout
errstou
		jsr convertname

		lda #OUTPUT_LFN
		ldx mydrive
		ldy #1
		jsr kernal.setlfs

		ldx #-1
-		inx
		lda name,x
		bne -
		txa
		ldx #<name
		ldy #>name
		jsr kernal.setnam
		jsr kernal.open

		ldx #INPUT_LFN
		jsr kernal.chkin

		jsr dolzo

		jsr kernal.clrchn
		lda #OUTPUT_LFN
		jsr kernal.close
		lda #INPUT_LFN
		jmp kernal.close

;-----------------------------------
convertname	.proc
		ldy #0
-		lda header.name,y
		beq +
		cmp #":"
		beq kesz
		iny
		cpy #200
		blt -
+
		lda #"/"
-		dey
		cmp header.name,y
		beq -			;skip trailing "/"
		iny
		lda #0
		sta header.name,y

		lda #"/"
-		dey
		beq ve
		cmp header.name,y
		bne -			;any "/"?
		sta name
		lda #":"
		sta name+2,y
		tya
		tax
-		lda header.name,y
		sta name+1,y
		dey
		cpy #255
		bne -
-		inx
		cpx #200
		beq +
		lda header.name,x
		sta name+2,x
		bne -
+		lda #0
		sta name+2,x
		geq kesz

ve		lda header.name,y	;normal filename
		sta name+1,y
		beq +
		iny
		cpy #200
		blt ve
+		lda #0
		sta name+1,y
		lda #":"
		sta name
kesz		rts
		.pend
;-----------------------------------
nametopetscii	.proc
		ldx #0
		ldy #-1
-		iny
		jsr utf8.topetscii
		cmp #32
		bge +
		cmp #1
		blt +
		lda #32
+		sta header.name,y
		bge -
		rts

utf8.parse.get	lda header.name,x
		beq +
		inx
+		rts
		.pend

utf8		.binclude "lib/utf8.asm"
;-----------------------------------
		.section bss
header		.block
magic		.fill 9
version		.word ?
libver		.word ?
versionneeded	.word ?
method		.byte ?
level		.byte ?
flags		.dword ?
filter		.dword ?
mode		.dword ?
mtime		.dword ?
gmtdiff		.dword ?
namesize	.fill 1
name		.fill 256
		.bend

header2		.block
checksum	.dword ?
filesize	.dword ?
compsize	.dword ?
checksumu	.dword ?
checksumc	.dword ?
		.bend

name		.fill 256
		.send bss
		.pend
		.send code
